#!/usr/bin/env python

"""Tests for `acdh_arche_pyutils.client` module."""
import os
import unittest
from acdh_arche_pyutils.client import ArcheApiClient


class Test_pyutils_client(unittest.TestCase):
    """Tests for `acdh_arche_pyutils.client` module."""

    def setUp(self):
        """Set up test fixtures, if any."""
        self.endpoint = "https://arche-curation.acdh-dev.oeaw.ac.at/api/"
        self.arche_client = ArcheApiClient(self.endpoint, out_dir='./out')
        self.top_col = self.arche_client.top_col_ids()

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_001_parse_url(self):
        self.assertEqual(self.arche_client.endpoint, self.endpoint)

    def test_002_get_description(self):
        description = self.arche_client.description
        self.assertEqual(type(description), dict)

    def test_003_populate_client_props(self):
        for x in ['id', 'parent', 'modification_date']:
            self.assertTrue(hasattr(self.arche_client, x))

    def test_004_base_url_match(self):
        a_cl = self.arche_client
        self.assertEqual(a_cl.endpoint, a_cl.fetched_endpoint)

    def test_005_top_cols(self):
        self.assertEqual(type(self.top_col), list)
        self.assertEqual(len(self.top_col[0]), 2)

    def test_006_get_resource(self):
        some_uri = self.top_col[0][0]
        res = self.arche_client.get_resource(some_uri)
        self.assertTrue("rdfg:Graph" in f"{res}")

    def test_007_write_resource_to_file(self):
        some_uri = self.top_col[0][0]
        res = self.arche_client.write_resource_to_file(some_uri)
        self.assertTrue(os.path.isfile(res))
        res_xml = self.arche_client.write_resource_to_file(some_uri, format='xml')
        self.assertTrue(res_xml.endswith('.xml'))
        os.remove(res)
        os.remove(res_xml)



#!/usr/bin/env python

"""Tests for `acdh_arche_pyutils.utils` module."""

import unittest
from acdh_arche_pyutils.utils import (
    camel_to_snake,
    create_query_sting,
    id_from_uri
)


class Test_pyutils_client(unittest.TestCase):
    """Tests for `acdh_arche_pyutils.utils` module."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_001_convert_camel_case(self):
        strings = [
            'CamelCase',
            'camel_Case',
            'camel_case',
        ]
        should_be = 'camel_case'
        for x in strings:
            self.assertEqual(camel_to_snake(x), should_be)

    def test_002_create_query_sting(self):
        query_params = {
            "p[0]": "http://www.w3.org/syntax-ns#type",
            "v[0]": "https://schema#TopCollection"
        }
        should_be = 'p[0]=http://www.w3.org/syntax-ns%23type&v[0]=https://schema%23TopCollection'
        self.assertEqual(create_query_sting(query_params), should_be)

    def test_003_id_from_uri(self):
        uris = [
            ['https://schema#TopCollection/123', '123'],
            ['https://schema#TopCollection/123/', '123'],
            ['https://schema#TopCollection/', '']
        ]
        for x in uris:
            self.assertEqual(
                id_from_uri(x[0]),
                x[1],
                f'{x[0]} should become {x[1]}'
            )



import rdflib
import requests
import yaml
import os
from acdh_arche_pyutils.utils import (
    camel_to_snake,
    create_query_sting,
    id_from_uri
)


class ArcheApiClient():
    """Main Class to interact with ARCHE-API """

    def __init__(
        self,
        arche_endpoint,
        out_dir='.'
    ):
        """ initializes the class
        :param arche_endpoint: The ARCHE endpoint e.g. `https://arche-dev.acdh-dev.oeaw.ac.at/api/`
        :type endpoint: str

        :param out_dir: a path to serialize data to, defaults to '.'
        :type out_dir: str

        :return: A ArcheApiClient instance
        :rtype: class:`achd_arch_pyutils.client.ArcheApiClient`
        """
        super().__init__()
        self.endpoint = arche_endpoint
        self.out_dir = out_dir
        self.describe_url = f"{arche_endpoint}describe"
        print(f'Fetching description for endpoint: {self.endpoint}')
        self.info = requests.get(self.describe_url)
        self.description = yaml.load(self.info.text, Loader=yaml.FullLoader)
        self.rest = self.description['rest']
        self.schema = self.description['schema']
        self.base_url = self.rest['urlBase']
        self.path_base = self.rest['pathBase']
        self.fetched_endpoint = f"{self.base_url}{self.path_base}"
        for key, value in self.schema.items():
            if isinstance(value, str):
                setattr(self, camel_to_snake(key), value)
        for key, value in self.schema['classes'].items():
            if isinstance(value, str):
                setattr(self, camel_to_snake(key), value)

    def top_col_ids(self):
        """returns of list of tuples (hasIdentifier, hasTitle) of all TopCollection"""
        query_params = {
            "property[0]": "http://www.w3.org/1999/02/22-rdf-syntax-ns#type",
            "value[0]": self.top_collection,
            "readMode": 'ids'
        }
        query_string = create_query_sting(query_params)
        r = requests.get(f"{self.fetched_endpoint}search?{query_string}")
        g = rdflib.Graph().parse(data=r.text, format='ttl')
        items = [
            (
                str(x[0]),
                str(x[1])
            ) for x in g.subject_objects(predicate=rdflib.URIRef(self.label))
        ]
        return items

    def get_resource(self, res_uri):
        """ fetches the given resource and its ancestors/parents
        :param res_uri: an ARCHE URI
        :type res_uri: str

        :return: A `rdflib.Graph` object
        :rtype: rdflib.Graph
        """
        query_params = {
            "readMode": "relatives",
            "parents[0]": self.parent,
        }
        query_string = create_query_sting(query_params)
        url = f"{res_uri}/metadata?{query_string}"
        print(f"fetching data for URI: {res_uri}, calling endpoint \n {url}")
        r = requests.get(url)
        g = rdflib.Graph().parse(data=r.text, format='ttl')
        return g

    def write_resource_to_file(self, res_uri, format='ttl'):
        """
        writes a resource (and its parents/children) to file on disk
        :param res_uri: An ARCHE-URI
        :type res_uri: str
        :param format: The serialisation format, defaults to 'ttl' -> turtle\
            use 'xml' for RDF/XML
        :type format: str

        :return: The location of the file
        :rtype: str
        """
        file_name = f"{id_from_uri(res_uri)}.{format}"
        save_path = os.path.join(self.out_dir, file_name)
        if format == 'ttl':
            format = 'turtle'
        else:
            format = 'xml'
        os.makedirs(os.path.dirname(save_path), exist_ok=True)
        g = self.get_resource(res_uri)
        g.serialize(save_path, format=format, encoding='utf8')
        return save_path



"""Some utility functions module."""


def camel_to_snake(s):
    """ converts CamelCase string to camel_case\
        taken from https://stackoverflow.com/a/44969381

        :param s: some string
        :type s: str:

        :return: a camel_case string
        :rtype: str:
    """
    no_camel = ''.join(['_'+c.lower() if c.isupper() else c for c in s]).lstrip('_')
    return no_camel.replace('__', '_')


def create_query_sting(param_dict):
    """ turns a dict into a query string
    :param param_dict: a dictionary
    :type param_dict: dict

    :return: a clean query string
    :rtype: str
    """
    params = "&".join(
        [
            f"{key}={value}" for key, value in param_dict.items()
        ]
    )
    return params.replace('#', '%23')


def id_from_uri(uri):
    """ extracts the id from an ARCHE-URL like https://whatever.com/123 -> 123
    :param uri: some ARCHE-URL
    :type uri: str

    :return: the actual ID, e.g. 123
    :rtype: str:
    """
    if uri.endswith('/'):
        uri = uri[:-1]
    a_id = uri.split('/')[-1]
    try:
        return f"{int(a_id)}"
    except ValueError:
        return ""
